@@ -1,3 +1,4 @@
+from QuantLib.QuantLib import OptionletVolatilityStructureHandle
 from fpql.termstructure import qlFlatForward, qlPiecewise
 import QuantLib as ql 
 from .pymodels import *
@@ -220,16 +221,19 @@ def get_qlcallablebond(bond_info: FixedRateBond,
     return callbond
 
 
-def callbond_stats(value_date: str,
-                    call_class: CallableBondParam, 
-                    ql_calendar,
-                    clean_price: float,
-                    engine: ql.QuantLib.TreeCallableFixedRateBondEngine,
-                    curve: Optional[ql.QuantLib.YieldTermStructureHandle] = None):
+def callbond_stats(**params):
+                    #clean_price: float,
+                    #engine: ql.QuantLib.TreeCallableFixedRateBondEngine,
+                    #curve: Optional[ql.QuantLib.YieldTermStructureHandle] = None):
     
-    vdate = datestr_to_qldate(value_date)
+    vdate = datestr_to_qldate(params.get('value_date'))
+    call_class = params.get('callable_bond_param')
+    ql_calendar = params.get('qlcalendar')
     ql_callbond = qlCallableBond_from_Class(call_class, ql_calendar)
-    if not curve:
+
+    if params.get("curve"):
+        curve_handle = params['curve']
+    else:
         if call_class.use_flat_curve:
             if call_class.flat_curve:
                 ffwd = qlFlatForward(call_class.flat_curve)
@@ -237,46 +241,58 @@ def callbond_stats(value_date: str,
         else:
             pwc = qlPiecewise(call_class.piecewise_curve, ql_calendar)
             curve_handle = ql.YieldTermStructureHandle(pwc)
+    
+    engine = params.get('engine')
+    if engine:
+        ql_callbond.setPricingEngine(engine)
     else:
-        curve_handle = curve
-
-    ql_callbond.setPricingEngine(engine)
+        return {}
+    
     call_risks = {}
     
-    if clean_price is None:
+    if params.get('clean_price'):
+        c_price = params['clean_price']
+    else:
         c_price = ql_callbond.cleanPrice()
         call_risks['Clean Price'] = c_price
-    else:
-        c_price = clean_price
-    oas = ql_callbond.OAS(c_price, 
-                        curve_handle,
-                        ql_day_count[call_class.info.settings.day_count],
-                        ql.Compounded,
-                        ql_frequency[call_class.info.settings.frequency],
-                        vdate) * 100
-    call_risks["OAS"] =  oas
+    oas = None
+
+    try:
+        oas = ql_callbond.OAS(c_price, 
+                            curve_handle,
+                            ql_day_count[call_class.info.settings.day_count],
+                            ql.Compounded,
+                            ql_frequency[call_class.info.settings.frequency],
+                            vdate) * 100
+        call_risks["OAS"] =  oas
+        
+    except:
+        pass
+
+    
     bond_yield  = ql_callbond.bondYield(c_price,
                         ql_day_count[call_class.info.settings.day_count],
                         ql.Compounded,
                         ql_frequency[call_class.info.settings.frequency],
                         vdate) * 100
     call_risks["Yield"] = bond_yield
-
-    e_convex = ql_callbond.effectiveConvexity(oas/100, 
-                                curve_handle,
-                                ql_day_count[call_class.info.settings.day_count],
-                                ql.Compounded,
-                                ql_frequency[call_class.info.settings.frequency])
-    call_risks["Effective Convexity"] = e_convex
-
-    e_duration = ql_callbond.effectiveDuration(oas/100, 
-                                curve_handle,
-                                ql_day_count[call_class.info.settings.day_count],
-                                ql.Compounded,
-                                ql_frequency[call_class.info.settings.frequency])
-    call_risks["Effective Duration"] = e_duration
     
-    #Calculate Yield To Call
+    #OAS/Spread is not supported for trees other than OneFactorModel
+    if oas is not None:
+        e_convex = ql_callbond.effectiveConvexity(oas/100, 
+                                    curve_handle,
+                                    ql_day_count[call_class.info.settings.day_count],
+                                    ql.Compounded,
+                                    ql_frequency[call_class.info.settings.frequency])
+        call_risks["Effective Convexity"] = e_convex
+
+        e_duration = ql_callbond.effectiveDuration(oas/100, 
+                                curve_handle,
+                                ql_day_count[call_class.info.settings.day_count],
+                                ql.Compounded,
+                                ql_frequency[call_class.info.settings.frequency])
+        call_risks["Effective Duration"] = e_duration
+
 
     return call_risks
 
