@@ -12,7 +12,8 @@ from numpy import datetime64 as dt64
 class Rate(BaseModel):
     tenor: str
     rate: float
-    start_basis: StartBasis = StartBasis.today
+    start_basis: Optional[StartBasis] = None
+    eom: Optional[bool] = None
 
     @validator('rate')
     def valid_rate(cls, v):
@@ -24,6 +25,33 @@ class Rate(BaseModel):
         return  v
 
 
+class FRA(BaseModel):
+    depo: Rate
+    start_tenor: Optional[str] = None
+    end_tenor: Optional[str] = None
+    start_date: Optional[datetime.date] = None
+    end_date: Optional[datetime.date] = None
+    eom: bool = True
+
+
+class Swap(BaseModel):
+    pass
+
+
+class IborFamily(BaseModel):
+    name: str
+    tenors: List[str]
+    start_basis: StartBasis = StartBasis.today
+    currency: str 
+    fixing_calendar: Optional[str] = None
+    business_day: BusinessDay
+    day_count: DayCount
+    eom: bool = True
+
+
+
+
+
 class Vol(BaseModel):
     date: str
     rate: float
@@ -61,9 +89,9 @@ class BondSetting(BaseModel):
     frequency: Frequency = Frequency.semi_annual
     start_basis: StartBasis = StartBasis.today
     day_count: DayCount = DayCount.act_act_act365
-    #pmt_day_count: DayCount = DayCount.act_act_act365
+    pmt_business_day: Optional[BusinessDay] = None
     business_day: BusinessDay = BusinessDay.no_adjustment
-    terminate_business_day: BusinessDay = BusinessDay.no_adjustment
+    terminate_business_day: Optional[BusinessDay] = None
     date_gen: DateGeneration = DateGeneration.backward
     is_eom: bool = False
 
